@@ -140,20 +140,21 @@ arma::sp_mat rewire_endpoints(
   arma::sp_mat newgraph(graph);
 
   // Getting the indexes
-  arma::umat indexes = sparse_indexes(graph);
 
-  for (unsigned i= 0;i<indexes.n_rows; i++) {
+// for (unsigned i= 0;i<indexes.n_rows; i++) {
+  unsigned int i = 0;
+  for (arma::sp_mat::const_iterator it = graph.begin(); it != graph.end(); it++) {
 
     // Checking user interrupt
-    if (i % 1000 == 0)
+    if (++i % 1000 == 0)
       Rcpp::checkUserInterrupt();
 
     // Checking whether to change it or not
     if (unif_rand() > p) continue;
 
     // Indexes
-    int j = indexes.at(i, 0);
-    int k = indexes.at(i, 1);
+    int j = it.row();
+    int k = it.col();
 
     // In the case of undirected graphs, we only modify the lower triangle
     // The upper triangle part will be rewritten during the rand.
@@ -200,8 +201,22 @@ arma::sp_mat rewire_swap(
 
   // Getting the indexes
   arma::umat indexes(graph.n_nonzero, 2);
-  if (undirected) indexes = sparse_indexes(sp_trimatl(newgraph));
-  else            indexes = sparse_indexes(newgraph);
+  unsigned int m = 0;
+  for (arma::sp_mat::const_iterator it = newgraph.begin(); it != newgraph.end(); it++) {
+
+    // Checking cases
+    if      (!self      && (it.row() == it.col())) continue;
+    else if (undirected && (it.row() <  it.col())) continue;
+
+    // Filling the matrix
+    indexes.at(m,0) = it.row();
+    indexes.at(m++,1) = it.col();
+
+  }
+
+  // Shedding rows
+  if (m < indexes.n_rows)
+    indexes.shed_rows(m, indexes.n_rows - 1u);
 
   // double dens = graph.n_nonzero/(graph.n_cols*graph.n_cols);
   int s = 0;
@@ -343,12 +358,10 @@ set.seed(1133)
 x <- barabasi.game(1e4)
 y <- as_adj(x)
 
-ind <- netdiffuseR:::sparse_indexes(y)
 
 microbenchmark(
   ig      = rewire(x, keeping_degseq(niter = 100)),
   nd      = netdiffuseR:::rewire_swap(y, 100),
-  # nd_fast = netdiffuseR:::rewire_swap_fast(y, ind, 100),
   unit="relative"
 )
 
@@ -490,24 +503,6 @@ arma::sp_mat permute_graph_cpp(const arma::sp_mat & x,
 //
 // -----------------------------------------------------------------------------
 
-// Checking if integer i is in x.
-int does_exists(const std::vector< unsigned int > & x, unsigned int val) {
-  int ans = -1;
-
-  // If the vector is empty, then nothing to see here
-  if (x.size() == 0)
-    return ans;
-
-  for (unsigned int i = 0; i < x.size(); i++) {
-    if (x.at(i) != val) continue;
-
-    ans = i;
-    break;
-  }
-
-  return ans;
-}
-
 // [[Rcpp::export]]
 arma::sp_mat rgraph_ba_cpp(
     const arma::sp_mat & graph,
@@ -524,14 +519,12 @@ arma::sp_mat rgraph_ba_cpp(
   dgr_new.subvec(0, m0-1) = dgr;
 
   std::vector< std::vector<unsigned int> > source(n);
-  std::vector< std::vector<double> > value(n);
 
   // Setting the initial values
   int nlocations = graph.n_nonzero;
   arma::sp_mat::const_iterator iter;
   for (iter = graph.begin(); iter != graph.end(); iter++) {
-    source.at(iter.row()).push_back(iter.col());
-    value.at(iter.row()).push_back(*iter);
+    source[iter.row()].push_back(iter.col());
   }
 
 
@@ -542,7 +535,7 @@ arma::sp_mat rgraph_ba_cpp(
   double randdraw, cump;
 
   // If self=true, then the prob are computed over m0+1, otherwise only over m0
-  int extra = self? 1 : 0, exists;
+  int extra = self? 1 : 0;
   for(int i=0;i<t;i++) {
     // Checling user interrup
     if (i % 1000 == 0)
@@ -581,15 +574,9 @@ arma::sp_mat rgraph_ba_cpp(
 
         // Links to the set of previous vertices
         if (randdraw <= cump) {
-          exists = does_exists(source.at(m0), k);
 
-          if (exists < 0) {
-            source.at(m0).push_back(k);
-            value.at(m0).push_back(1.0);
-            ++nlocations;
-          } else {
-            ++value.at(m0).at(exists);
-          }
+          source[m0].push_back(k);
+          ++nlocations;
 
           dgr_new.at(k) += 1.0;
 
@@ -604,25 +591,30 @@ arma::sp_mat rgraph_ba_cpp(
 
   // Now need to coerce into a -sp_mat-
   arma::umat locations(2, nlocations);
-  arma::colvec values(nlocations);
+  arma::colvec values(nlocations, arma::fill::ones);
 
   int curloc = 0;
   for (int i = 0; i<n; i++) {
-    for (unsigned int j = 0; j < source.at(i).size(); j++) {
+    for (unsigned int j = 0; j < source[i].size(); j++) {
       locations.at(0, curloc) = i;
-      locations.at(1, curloc) = source.at(i).at(j);
-      values.at(curloc++) = value.at(i).at(j);
+      locations.at(1, curloc++) = source[i][j];
     }
 
   }
 
   // Creating the sparse matrix
-  arma::sp_mat graph_new(locations, values, n, n, true, false);
+  arma::sp_mat graph_new(true, locations, values, n, n, true, false);
 
   return graph_new;
 }
 
 
+// The bag algorithm should be straight forward.
+// For each time that node i receives a link, add it to the bag, which we
+// know what is the size in the beginning.
+// Later, to draw a new vertex, it suffices to bag[runif()*length(bag)],
+// so we get him from the bag.
+
 // [[Rcpp::export]]
 arma::sp_mat rgraph_ba_new_cpp(int m0 = 1, int m = 1, int t = 10, bool self=true) {
   int n = m0;
